#include <plib.h>
#include <stdint.h>
#include "UART.h"
#include "LED.h"

//! Initialize the pins for switching the TRIAC
void SW_pin_Init(void){
    TRISBbits.TRISB7 = 0;
    TRISBbits.TRISB8 = 0;

    VDD = 1;
    SWITCH = SW_OFF;
}

//! Read the peak power and send the value over UART.
/*! This is actually only an estimate since it uses RMS values.
 * The actual power would require the real-time multiplication
 * of the voltage and current waveforms (especially for inductive loads).
 *
 * Reading 0 from the ADC is 0V and 1023 from the ADC is 3.3V.
 * Since the output from the current transformer is biased at 1.65V,
 * we subtract that from the voltage read from the ADC to find the actual
 * voltage generated across the transformer and divide by sqrt(2) to find RMS.
 * The AC equivalent resistance seen by the current transformer when the 3.3V
 * source is shorted is 298.211 ohms (due to the 300 ohm and two 100kohm).
 * Then using the datasheet (30A in produces 15mA out) we can find the current.
 * The power is then estimated as the RMS voltage (~120) times the RMS current.
 */
void send_power(void) {
    int ADCPeak;

    ADCPeak = ReadADCPeak();

    double adc_V, v_RMS, i_RMS, power;

    adc_V = (V_BBB3V3/1023)*ADCPeak;     //Converts digital value to V value

    v_RMS = (adc_V - V_BBB3V3/2) / 1.41421;  // Calculate Vrms of input voltage

    i_RMS = v_RMS * 6.7067;   //Based on data sheet 15mA/30A
                                //I = V/298.211*(15mA/30A)
                                //I = 6.7067*V

    power = i_RMS * V_MAINS_RMS;      //Voltage assumed to be 116 volts

    printf("%05.2f\n",power);
}

//! Read the status of the switch pin and send over UART
void send_status(void) {

    printf("%i",(SWITCH == SW_ON));
}

//! Initialize UART functionality on the PIC
void UART_Init(void) {
    // map U2RX to RPB11
    U2RXR = 0x03;

    // map pin RPB10 to U2TX
    RPB10R = 0x02;

    // Initalize all settings to default by setting UxMODE and UxSTA to 0
    U2MODE = 0;
    U2STA = 0;

    // Set the baud rate to 9600bps
    U2BRG = 1000000 / (4 * BAUD_RATE) - 1;

    // Clear the UART receive interrupt flag
    IFS1bits.U2RXIF = 0;

    // Set the UART receive interrupt priority to 4
    IPC9bits.U2IP = 4;

    // Turn on high speed BRG
    U2MODEbits.BRGH = 1;

    // Turn the UART module on
    U2MODEbits.ON = 1;

    // Enable the UART TX module
    U2STAbits.UTXEN = 1;

    // Enable the UART RX module
    U2STAbits.URXEN = 1;

    // Enable UART RX interrupts
    IEC1bits.U2RXIE = 1;

}

//! Executes UART interrupt on the PIC (BBB communication)
/*! Receiving R sends a new power reading over UART.
 * Receiving S sends the switch pin status over UART.
 * Receiving N turns the TRIAC on.
 * Receiving F turns the TRIAC off.
 */
void __ISR(_UART_2_VECTOR) UART_Interrupt(void) {
    uint8_t command;

    // check if UART RX interrupt flag is set
    if (IFS1bits.U2RXIF) {
        // while data is available to be read (UxSTAbits.URXDA)
        while (U2STAbits.URXDA) {
            // the first byte represents the command
            command = U2RXREG;

            switch (command) {
                // send current value over UART
                case 'R': send_power(); break;
                
                // send on/off status over UART
                case 'S': send_status(); break;

                // set on
                case 'N': SWITCH = SW_ON; break;

                // set off
                case 'F': SWITCH = SW_OFF; break;
            }
            // clear the interrupt flag
            IFS1bits.U2RXIF = 0;
        }
    }
}
